@@ -1,28 +1,29 @@
-/* -*- mode: c; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ex: set tabstop=2 softtabstop=2 shiftwidth=2 expandtab: */
-
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009,2011,2012, Intel Corporation.
+ * Copyright (C) 2022-2023 Intel Corporation.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms and conditions of the GNU General Public License,
- * version 2, as published by the Free Software Foundation.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 
- * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
 #ifndef _AHCI_H_INCLUDED_
 #define _AHCI_H_INCLUDED_
 
+#include "block.h"
+#include "ibpi.h"
+
 /**
  * @brief Gets sysfs path to SATA port.
  *
@@ -51,4 +52,4 @@ char *ahci_get_port_path(const char *path);
  */
 int ahci_sgpio_write(struct block_device *path, enum ibpi_pattern ibpi);
 
-#endif /* _AHCI_H_INCLUDED_ */
+#endif				/* _AHCI_H_INCLUDED_ */
