@@ -1,19 +1,20 @@
 /*
  * AMD IPMI LED control
- * Copyright (C) 2021, Advanced Micro Devices, Inc.
+ * Copyright (C) 2023, Advanced Micro Devices, Inc.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms and conditions of the GNU General Public License,
- * version 2, as published by the Free Software Foundation.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -23,6 +24,7 @@
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <string.h>
 #include <unistd.h>
 #include <libgen.h>
@@ -43,15 +45,28 @@
 #include "amd.h"
 #include "ipmi.h"
 
-static uint8_t amd_ibpi_ipmi_register[] = {
-	[IBPI_PATTERN_PFA] = 0x41,
-	[IBPI_PATTERN_LOCATE] = 0x42,
-	[IBPI_PATTERN_FAILED_DRIVE] = 0x44,
-	[IBPI_PATTERN_FAILED_ARRAY] = 0x45,
-	[IBPI_PATTERN_REBUILD] = 0x46,
-	[IBPI_PATTERN_HOTSPARE] = 0x47,
+/* For IBPI_PATTERN_NORMAL and IBPI_PATTERN_ONESHOT_NORMAL _disable_all_ibpi_states is called. */
+const struct ibpi2value ibpi2amd_ipmi[] = {
+	{IBPI_PATTERN_PFA, 0x41},
+	{IBPI_PATTERN_LOCATE, 0x42},
+	{IBPI_PATTERN_FAILED_DRIVE, 0x44},
+	{IBPI_PATTERN_FAILED_ARRAY, 0x45},
+	{IBPI_PATTERN_REBUILD, 0x46},
+	{IBPI_PATTERN_HOTSPARE, 0x47},
+	{IBPI_PATTERN_UNKNOWN}
 };
 
+#define MG9098_CHIP_ID_REG	0x63
+
+#define AMD_IPMI_NETFN		0x06
+#define AMD_IPMI_CMD		0x52
+
+#define AMD_ETHANOL_X_CHANNEL	0x0d
+#define AMD_DAYTONA_X_CHANNEL	0x17
+
+#define AMD_BASE_SLAVE_ADDR	0xc0
+#define AMD_NVME_SLAVE_ADDR	0xc4
+
 /* The path we are given should be similar to
  * /sys/devices/pci0000:e0/0000:e0:03.3/0000:e3:00.0
  *                                      ^^^^^^^^^^
@@ -95,7 +110,8 @@ static int _get_ipmi_nvme_port(char *path)
 			char *dname = strrchr(dir_path, '/');
 
 			dname++;
-			port = strtol(dname, NULL, 0);
+			if (str_toi(&port, dname, NULL, 0) != 0)
+				return -1;
 			break;
 		}
 	}
@@ -148,7 +164,8 @@ static int _get_ipmi_sata_port(const char *start_path)
 
 	/* skip past 'ata' to get the ata port number */
 	t += 3;
-	port = strtoul(t, NULL, 10);
+	if (str_toi(&port, t, NULL, 10) != 0)
+		return -1;
 
 	return port;
 }
@@ -204,10 +221,10 @@ static int _ipmi_platform_channel(struct amd_drive *drive)
 
 	switch (amd_ipmi_platform) {
 	case AMD_PLATFORM_ETHANOL_X:
-		drive->channel =  0xd;
+		drive->channel = AMD_ETHANOL_X_CHANNEL;
 		break;
 	case AMD_PLATFORM_DAYTONA_X:
-		drive->channel = 0x17;
+		drive->channel = AMD_DAYTONA_X_CHANNEL;
 		break;
 	default:
 		rc = -1;
@@ -224,27 +241,27 @@ static int _ipmi_platform_slave_address(struct amd_drive *drive)
 
 	switch (amd_ipmi_platform) {
 	case AMD_PLATFORM_ETHANOL_X:
-		drive->slave_addr = 0xc0;
+		drive->slave_addr = AMD_BASE_SLAVE_ADDR;
 		break;
 	case AMD_PLATFORM_DAYTONA_X:
 		if (drive->dev == AMD_NO_DEVICE) {
 			/* Assume base slave address, we may not be able
 			 * to retrieve a valid amd_drive yet.
 			 */
-			drive->slave_addr = 0xc0;
+			drive->slave_addr = AMD_BASE_SLAVE_ADDR;
 		} else if (drive->dev == AMD_NVME_DEVICE) {
 			/* On DaytonaX systems only drive bays 19 - 24
 			 * support NVMe devices so use the slave address
 			 * for the corresponding MG9098 chip.
 			 */
-			drive->slave_addr = 0xc4;
+			drive->slave_addr = AMD_NVME_SLAVE_ADDR;
 		} else {
 			if (drive->port <= 8)
-				drive->slave_addr = 0xc0;
+				drive->slave_addr = AMD_BASE_SLAVE_ADDR;
 			else if (drive->port > 8 && drive->port < 17)
-				drive->slave_addr = 0xc2;
+				drive->slave_addr = AMD_BASE_SLAVE_ADDR + 2;
 			else
-				drive->slave_addr = 0xc4;
+				drive->slave_addr = AMD_NVME_SLAVE_ADDR;
 		}
 
 		break;
@@ -257,8 +274,7 @@ static int _ipmi_platform_slave_address(struct amd_drive *drive)
 	return rc;
 }
 
-static int _set_ipmi_register(int enable, uint8_t reg,
-			      struct amd_drive *drive)
+static int _set_ipmi_register(int enable, uint8_t reg, struct amd_drive *drive)
 {
 	int rc;
 	int status, data_sz;
@@ -285,8 +301,8 @@ static int _set_ipmi_register(int enable, uint8_t reg,
 	log_debug(REG_FMT_2, "channel", cmd_data[0], "slave addr", cmd_data[1]);
 	log_debug(REG_FMT_2, "len", cmd_data[2], "register", cmd_data[3]);
 
-	rc = ipmicmd(BMC_SA, 0x0, 0x6, 0x52, 4, &cmd_data, 1, &data_sz,
-		     &status);
+	rc = ipmicmd(BMC_SA, 0x0, AMD_IPMI_NETFN, AMD_IPMI_CMD, 4, &cmd_data,
+		     1, &data_sz, &status);
 	if (rc) {
 		log_error("Could not determine current register %x setting\n",
 			  reg);
@@ -310,8 +326,8 @@ static int _set_ipmi_register(int enable, uint8_t reg,
 	log_debug(REG_FMT_2, "len", cmd_data[2], "register", cmd_data[3]);
 	log_debug(REG_FMT_1, "status", cmd_data[4]);
 
-	rc = ipmicmd(BMC_SA, 0x0, 0x6, 0x52, 5, &cmd_data, 1, &data_sz,
-		     &status);
+	rc = ipmicmd(BMC_SA, 0x0, AMD_IPMI_NETFN, AMD_IPMI_CMD, 5, &cmd_data,
+		     1, &data_sz, &status);
 	if (rc) {
 		log_error("Could not enable register %x\n", reg);
 		return rc;
@@ -326,27 +342,33 @@ static int _enable_smbus_control(struct amd_drive *drive)
 	return _set_ipmi_register(1, 0x3c, drive);
 }
 
-static int _enable_ibpi_state(struct amd_drive *drive, enum ibpi_pattern ibpi)
+static int _change_ibpi_state(struct amd_drive *drive, enum ibpi_pattern ibpi, bool enable)
 {
-	log_debug("Enabling %s LED\n", ibpi2str(ibpi));
-	return _set_ipmi_register(1, amd_ibpi_ipmi_register[ibpi], drive);
-}
+	const struct ibpi2value *ibpi2val = get_by_ibpi(ibpi, ibpi2amd_ipmi,
+							ARRAY_SIZE(ibpi2amd_ipmi));
 
-static int _disable_ibpi_state(struct amd_drive *drive, enum ibpi_pattern ibpi)
-{
-	log_debug("Disabling %s LED\n", ibpi2str(ibpi));
-	return _set_ipmi_register(0, amd_ibpi_ipmi_register[ibpi], drive);
+	if (ibpi2val->ibpi == IBPI_PATTERN_UNKNOWN) {
+		log_info("AMD_IPMI: Controller doesn't support %s pattern\n", ibpi_str[ibpi]);
+		return STATUS_INVALID_STATE;
+	}
+
+	if (enable)
+		log_debug("Enabling %s LED\n", ibpi2str(ibpi));
+	else
+		log_debug("Disabling %s LED\n", ibpi2str(ibpi));
+
+	return _set_ipmi_register(enable, ibpi2val->value, drive);
 }
 
 static int _disable_all_ibpi_states(struct amd_drive *drive)
 {
 	int rc;
 
-	rc = _disable_ibpi_state(drive, IBPI_PATTERN_PFA);
-	rc |= _disable_ibpi_state(drive, IBPI_PATTERN_LOCATE);
-	rc |= _disable_ibpi_state(drive, IBPI_PATTERN_FAILED_DRIVE);
-	rc |= _disable_ibpi_state(drive, IBPI_PATTERN_FAILED_ARRAY);
-	rc |= _disable_ibpi_state(drive, IBPI_PATTERN_REBUILD);
+	rc = _change_ibpi_state(drive, IBPI_PATTERN_PFA, false);
+	rc |= _change_ibpi_state(drive, IBPI_PATTERN_LOCATE, false);
+	rc |= _change_ibpi_state(drive, IBPI_PATTERN_FAILED_DRIVE, false);
+	rc |= _change_ibpi_state(drive, IBPI_PATTERN_FAILED_ARRAY, false);
+	rc |= _change_ibpi_state(drive, IBPI_PATTERN_REBUILD, false);
 
 	return rc;
 }
@@ -368,19 +390,20 @@ int _amd_ipmi_em_enabled(const char *path)
 	cmd_data[0] = drive.channel;
 	cmd_data[1] = drive.slave_addr;
 	cmd_data[2] = 0x1;
-	cmd_data[3] = 0x63;
+	cmd_data[3] = MG9098_CHIP_ID_REG;
 
 	status = 0;
-	rc = ipmicmd(BMC_SA, 0x0, 0x6, 0x52, 4, &cmd_data, 1,
-		     &data_sz, &status);
+	rc = ipmicmd(BMC_SA, 0x0, AMD_IPMI_NETFN, AMD_IPMI_CMD, 4, &cmd_data,
+		     1, &data_sz, &status);
 
 	if (rc) {
 		log_error("Can't determine MG9098 Status for AMD platform\n");
 		return 0;
 	}
 
+	/* Status return of 98 indicates MG9098 backplane */
 	if (status != 98) {
-		log_error("Platform %s does not have a MG9098 controller\n");
+		log_error("Platform does not have a MG9098 controller\n");
 		return 0;
 	}
 
@@ -406,7 +429,7 @@ int _amd_ipmi_write(struct block_device *device, enum ibpi_pattern ibpi)
 	}
 
 	if (ibpi == IBPI_PATTERN_LOCATE_OFF) {
-		rc = _disable_ibpi_state(&drive, IBPI_PATTERN_LOCATE);
+		rc = _change_ibpi_state(&drive, IBPI_PATTERN_LOCATE, false);
 		return rc;
 	}
 
@@ -414,7 +437,7 @@ int _amd_ipmi_write(struct block_device *device, enum ibpi_pattern ibpi)
 	if (rc)
 		return rc;
 
-	rc = _enable_ibpi_state(&drive, ibpi);
+	rc = _change_ibpi_state(&drive, ibpi, true);
 	if (rc)
 		return rc;
 
@@ -423,20 +446,16 @@ int _amd_ipmi_write(struct block_device *device, enum ibpi_pattern ibpi)
 
 char *_amd_ipmi_get_path(const char *cntrl_path, const char *sysfs_path)
 {
-	char *p, *t;
+	char *t;
 
 	/* For NVMe devices we can just dup the path sysfs path */
-	p = strstr(cntrl_path, "nvme");
-	if (p)
+	if (strstr(cntrl_path, "nvme"))
 		return strdup(sysfs_path);
 
-	/* For SATA devices we need everything up to 'ataXX/' in the path */
-	p = strdup(cntrl_path);
-	if (!p)
-		return NULL;
-
-	/* Find the beginning of the ataXX piece of the path */
-	t = strstr(p, "ata");
+	/*
+	 * For SATA devices we need everything up to 'ataXX/' in the path
+	 */
+	t = strstr(cntrl_path, "ata");
 	if (!t)
 		return NULL;
 
@@ -447,8 +466,6 @@ char *_amd_ipmi_get_path(const char *cntrl_path, const char *sysfs_path)
 	if (!t)
 		return NULL;
 
-	*++t = '\0';
-
-	return p;
+	return strndup(cntrl_path, (t - cntrl_path) + 1);
 }
 
