@@ -1,19 +1,20 @@
 /*
  * AMD SGPIO LED control
- * Copyright (C) 2019, Advanced Micro Devices, Inc.
+ * Copyright (C) 2023, Advanced Micro Devices, Inc.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms and conditions of the GNU General Public License,
- * version 2, as published by the Free Software Foundation.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -275,7 +276,7 @@ static struct cache_entry *_get_cache(struct amd_drive *drive)
 	 *	cache_entry[n] => drive (4*n) to drive (4*n + 3)
 	 */
 
-	index = (drive->ata_port / 4);
+	index = ((drive->ata_port - 1) / 4);
 
 	return &sgpio_cache[index];
 }
@@ -569,7 +570,8 @@ static int _get_amd_sgpio_drive(const char *start_path,
 
 	/* skip past 'ata' to get the ata port number */
 	a += 3;
-	drive->ata_port = strtoul(a, NULL, 10);
+	if (str_toi(&drive->ata_port, a, NULL, 10) != 0)
+		return -1;
 
 	found = _find_file_path(ata_dir, "port_no", path, PATH_MAX);
 	if (!found) {
