@@ -1,19 +1,20 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009-2020 Intel Corporation.
+ * Copyright (C) 2022-2023 Intel Corporation.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms and conditions of the GNU General Public License,
- * version 2, as published by the Free Software Foundation.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -141,7 +142,7 @@ static char *_get_host(char *path, struct cntrl_device *cntrl)
 	char *result = NULL;
 
 	if (cntrl->cntrl_type == CNTRL_TYPE_SCSI)
-		result = scsi_get_slot_path(path, cntrl->sysfs_path);
+		result = scsi_get_host_path(path, cntrl->sysfs_path);
 	else if (cntrl->cntrl_type == CNTRL_TYPE_AHCI)
 		result = ahci_get_port_path(path);
 	else if (cntrl->cntrl_type == CNTRL_TYPE_DELLSSD)
@@ -152,7 +153,7 @@ static char *_get_host(char *path, struct cntrl_device *cntrl)
 		result = npem_get_path(cntrl->sysfs_path);
 	else if (cntrl->cntrl_type == CNTRL_TYPE_AMD)
 		result = amd_get_path(path, cntrl->sysfs_path);
-  
+
 	return result;
 }
 
@@ -190,7 +191,7 @@ struct cntrl_device *block_get_controller(const struct list *cntrl_list, char *p
 
 	list_for_each(cntrl_list, cntrl) {
 		if (strncmp(cntrl->sysfs_path, path,
-			    strlen(cntrl->sysfs_path)) == 0) {
+			    strnlen(cntrl->sysfs_path, PATH_MAX)) == 0) {
 			if (cntrl->cntrl_type == CNTRL_TYPE_NPEM)
 				return cntrl;
 			non_npem_cntrl = cntrl;
@@ -215,6 +216,24 @@ struct _host_type *block_get_host(struct cntrl_device *cntrl, int host_id)
 	return hosts;
 }
 
+struct block_device *get_block_device_from_sysfs_path(char *sub_path, bool sub_path_to_end)
+{
+	struct block_device *device;
+
+	list_for_each(sysfs_get_block_devices(), device) {
+			char *start_loc;
+			if ((start_loc = strstr(device->sysfs_path, sub_path))) {
+				char following = start_loc[strnlen(sub_path, PATH_MAX)];
+				if (following == '/' || following == '\n' || following == '\0')
+					return device;
+				if (sub_path_to_end)
+					return device;
+			}
+	}
+
+	return NULL;
+}
+
 /*
  * Allocates a new block device structure. See block.h for details.
  */
