@@ -1,19 +1,20 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009-2021 Intel Corporation.
+ * Copyright (C) 2022-2023 Intel Corporation.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms and conditions of the GNU General Public License,
- * version 2, as published by the Free Software Foundation.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -38,6 +39,7 @@
 #include "utils.h"
 #include "amd.h"
 #include "npem.h"
+#include "vmdssd.h"
 
 /**
  * @brief Name of controllers types.
@@ -55,6 +57,21 @@ static const char * const ctrl_type_str[] = {
 	[CNTRL_TYPE_AMD]     = "AMD",
 };
 
+enum cntrl_type string_to_cntrl_type(const char *cntrl_str)
+{
+	for(int i = 0; i < ARRAY_SIZE(ctrl_type_str); i++) {
+		if (strcasecmp(cntrl_str, ctrl_type_str[i]) == 0 ) {
+			return (enum cntrl_type)i;
+		}
+	}
+	return CNTRL_TYPE_UNKNOWN;
+}
+
+const char * const cntrl_type_to_string(enum cntrl_type cntrl)
+{
+	return ctrl_type_str[cntrl];
+}
+
 /**
  */
 static int _is_storage_controller(const char *path)
@@ -119,10 +136,12 @@ static int _is_amd_nvme_cntrl(const char *path)
 	char tmp[PATH_MAX];
 	char *t;
 
+	memset(&tmp, 0, sizeof(tmp));
+
 	if (!_is_nvme_cntrl(path))
 		return 0;
 
-	sprintf(tmp, "%s", path);
+	strncpy(tmp, path, PATH_MAX - 1);
 	t = strrchr(tmp, '/');
 	if (!t)
 		return 0;
@@ -395,24 +414,24 @@ struct cntrl_device *cntrl_device_init(const char *path)
 
 	type = _get_type(path);
 	if (type != CNTRL_TYPE_UNKNOWN) {
-		if (!list_is_empty(&conf.cntrls_whitelist)) {
+		if (!list_is_empty(&conf.cntrls_allowlist)) {
 			char *cntrl = NULL;
 
-			list_for_each(&conf.cntrls_whitelist, cntrl) {
+			list_for_each(&conf.cntrls_allowlist, cntrl) {
 				if (match_string(cntrl, path))
 					break;
 				cntrl = NULL;
 			}
 			if (!cntrl) {
-				log_debug("%s not found on whitelist, ignoring", path);
+				log_debug("%s not found on allowlist, ignoring", path);
 				return NULL;
 			}
-		} else if (!list_is_empty(&conf.cntrls_blacklist)) {
+		} else if (!list_is_empty(&conf.cntrls_excludelist)) {
 			char *cntrl;
 
-			list_for_each(&conf.cntrls_blacklist, cntrl) {
+			list_for_each(&conf.cntrls_excludelist, cntrl) {
 				if (match_string(cntrl, path)) {
-					log_debug("%s found on blacklist, ignoring",
+					log_debug("%s found on excludelist, ignoring",
 						  path);
 					return NULL;
 				}
@@ -435,7 +454,7 @@ struct cntrl_device *cntrl_device_init(const char *path)
 			em_enabled = 0;
 		}
 		if (em_enabled) {
-			device = malloc(sizeof(struct cntrl_device));
+			device = calloc(1, sizeof(struct cntrl_device));
 			if (device) {
 				if (type == CNTRL_TYPE_SCSI) {
 					device->isci_present = _is_isci_cntrl(path);
@@ -444,8 +463,15 @@ struct cntrl_device *cntrl_device_init(const char *path)
 					device->isci_present = 0;
 					device->hosts = NULL;
 				}
+				if (type == CNTRL_TYPE_VMD) {
+					char *domain = vmdssd_get_domain(path);
+
+					if (domain != NULL)
+						snprintf(device->domain, PATH_MAX,
+							 "%s", domain);
+				}
 				device->cntrl_type = type;
-				device->sysfs_path = str_dup(path);
+				strncpy(device->sysfs_path, path, PATH_MAX - 1);
 			}
 		} else {
 			log_error
@@ -463,7 +489,6 @@ struct cntrl_device *cntrl_device_init(const char *path)
 void cntrl_device_fini(struct cntrl_device *device)
 {
 	if (device) {
-		free(device->sysfs_path);
 		free_hosts(device->hosts);
 		free(device);
 	}
