@@ -1,25 +1,28 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009-2020 Intel Corporation.
+ * Copyright (C) 2022-2023 Intel Corporation.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms and conditions of the GNU General Public License,
- * version 2, as published by the Free Software Foundation.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
 #ifndef _CNTRL_H_INCLUDED_
 #define _CNTRL_H_INCLUDED_
 
+#include <limits.h>
+
 /**
  * This enumeration type lists all supported storage controller types.
  */
@@ -33,6 +36,23 @@ enum cntrl_type {
 	CNTRL_TYPE_AMD,
 };
 
+
+/**
+ * @brief Translates string to controller enumerated type
+ *
+ * @param cntrl_str          string to translate
+ * @return enum cntrl_type   enumerated value
+ */
+enum cntrl_type string_to_cntrl_type(const char *cntrl_str);
+
+/**
+ * @brief Translates enumerated type to string representation
+ *
+ * @param cntrl               Enumerated type to translate to string representation
+ * @return const char* const  String representation of enumerated type
+ */
+const char *const cntrl_type_to_string(enum cntrl_type cntrl);
+
 /**
  * @brief Storage controller device structure.
  *
@@ -42,7 +62,7 @@ struct cntrl_device {
 	/**
 	* Path to the device in sysfs tree.
 	*/
-	char *sysfs_path;
+	char sysfs_path[PATH_MAX];
 
 	/**
 	 * Type of storage controller device.
@@ -54,6 +74,11 @@ struct cntrl_device {
 	 */
 	int isci_present;
 
+	/**
+	 * Applicable to VMD controllers. Domain address.
+	 */
+	char domain[PATH_MAX];
+
 	struct _host_type {
 		/**
 		 * ibpi state buffer for directly attached devices
