@@ -1,7 +1,7 @@
 /*
  * Intel(R) Enclosure LED Utilities
  *
- * Copyright (C) 2017-2019 Intel Corporation.
+ * Copyright (C) 2017-2023 Intel Corporation.
  * Copyright (C) 2009 Karel Zak <kzak@redhat.com>
  *
  * SPDX-License-Identifier: GPL-2.0
@@ -44,9 +44,9 @@ struct ledmon_conf {
 	int rebuild_blink_on_all;
 	int raid_members_only;
 
-	/* whitelist and blacklist of controllers for blinking */
-	struct list cntrls_whitelist;
-	struct list cntrls_blacklist;
+	/* allowlist and excludelist of controllers for blinking */
+	struct list cntrls_allowlist;
+	struct list cntrls_excludelist;
 };
 
 extern struct ledmon_conf conf;
