@@ -1,25 +1,27 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009-2019 Intel Corporation.
+ * Copyright (C) 2022-2023 Intel Corporation.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms and conditions of the GNU General Public License,
- * version 2, as published by the Free Software Foundation.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
 
 #include <fcntl.h>
 #include <limits.h>
+#include <stdbool.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -37,18 +39,20 @@
 #include "enclosure.h"
 #include "ibpi.h"
 #include "list.h"
+#include "npem.h"
 #include "pci_slot.h"
 #include "raid.h"
 #include "slave.h"
+#include "slot.h"
 #include "stdio.h"
 #include "sysfs.h"
 #include "utils.h"
+#include "vmdssd.h"
 
 /**
  */
 #define SYSFS_CLASS_BLOCK       "/sys/block"
 #define SYSFS_CLASS_ENCLOSURE   "/sys/class/enclosure"
-#define SYSFS_PCI_DEVICES       "/sys/bus/pci/devices"
 #define SYSFS_PCI_SLOTS         "/sys/bus/pci/slots"
 
 /**
@@ -104,6 +108,14 @@ static struct list enclo_list;
  * function to initialize the variable. Use sysfs_scan() function to populate
  * the list. Use sysfs_reset() function to delete the content of the list.
  */
+static struct list pci_slots_list;
+
+/**
+ * This is internal variable global to sysfs module only. It is a list of
+ * all supported slots registered in the system. Use sysfs_init()
+ * function to initialize the variable. Use sysfs_scan() function to populate
+ * the list. Use sysfs_reset() function to delete the content of the list.
+ */
 static struct list slots_list;
 
 /**
@@ -240,6 +252,10 @@ static void _slave_cnt_add(const char *path, struct raid_device *raid)
 	struct slave_device *device;
 
 	char *t = strrchr(path, '/');
+
+	if (!t)
+		return;
+
 	if (strncmp(t + 1, "dev-", 4) == 0) {
 		device = slave_device_init(path, &sysfs_block_list);
 		if (device) {
@@ -343,11 +359,11 @@ static void _enclo_add(const char *path)
 
 /**
  */
-static void _slots_add(const char *path)
+static void _pci_slots_add(const char *path)
 {
 	struct pci_slot *device = pci_slot_init(path);
 	if (device)
-		list_append(&slots_list, device);
+		list_append(&pci_slots_list, device);
 }
 
 /**
@@ -355,6 +371,10 @@ static void _slots_add(const char *path)
 static void _check_raid(const char *path)
 {
 	char *t = strrchr(path, '/');
+
+	if (!t)
+		return;
+
 	if (strncmp(t + 1, "md", 2) == 0)
 		_raid_add(path);
 }
@@ -443,18 +463,50 @@ static void _scan_enclo(void)
 	}
 }
 
-static void _scan_slots(void)
+static void _scan_pci_slots(void)
 {
 	struct list dir;
 	if (scan_dir(SYSFS_PCI_SLOTS, &dir) == 0) {
 		const char *dir_path;
 
-		list_for_each(&dir, dir_path)
-			_slots_add(dir_path);
+		list_for_each(&dir, dir_path) {
+			if (vmdssd_check_slot_module(dir_path) == true)
+				_pci_slots_add(dir_path);
+		}
 		list_erase(&dir);
 	}
 }
 
+static void _scan_slots(void)
+{
+	struct pci_slot *pci_slot;
+	struct cntrl_device *cntrl_device;
+	struct enclosure_device *encl;
+	struct slot_property *slot;
+
+	list_for_each(sysfs_get_cntrl_devices(), cntrl_device) {
+		if (cntrl_device->cntrl_type == CNTRL_TYPE_NPEM) {
+			slot = npem_slot_property_init(cntrl_device);
+			if (slot)
+				list_append(&slots_list, slot);
+		}
+	}
+
+	list_for_each(sysfs_get_pci_slots(), pci_slot) {
+		slot = pci_slot_property_init(pci_slot);
+		if (slot)
+			list_append(&slots_list, slot);
+	}
+
+	list_for_each(sysfs_get_enclosure_devices(), encl) {
+		for (int i = 0; i < encl->slots_count; i++) {
+			slot = enclosure_slot_property_init(encl, i);
+			if (slot)
+				list_append(&slots_list, slot);
+		}
+	}
+}
+
 /**
  */
 static int _is_failed_array(struct raid_device *raid)
@@ -575,7 +627,8 @@ void sysfs_init(void)
 	list_init(&slave_list, (item_free_t)slave_device_fini);
 	list_init(&cntnr_list, (item_free_t)raid_device_fini);
 	list_init(&enclo_list, (item_free_t)enclosure_device_fini);
-	list_init(&slots_list, (item_free_t)pci_slot_fini);
+	list_init(&pci_slots_list, (item_free_t)pci_slot_fini);
+	list_init(&slots_list, NULL);
 }
 
 void sysfs_reset(void)
@@ -586,6 +639,7 @@ void sysfs_reset(void)
 	list_erase(&slave_list);
 	list_erase(&cntnr_list);
 	list_erase(&enclo_list);
+	list_erase(&pci_slots_list);
 	list_erase(&slots_list);
 }
 
@@ -593,9 +647,10 @@ void sysfs_scan(void)
 {
 	_scan_enclo();
 	_scan_cntrl();
-	_scan_slots();
+	_scan_pci_slots();
 	_scan_block();
 	_scan_raid();
+	_scan_slots();
 	_scan_slave();
 
 	_determine_slaves(&slave_list);
@@ -632,6 +687,11 @@ const struct list *sysfs_get_block_devices(void)
 }
 
 const struct list *sysfs_get_pci_slots(void)
+{
+	return &pci_slots_list;
+}
+
+const struct list *sysfs_get_slots(void)
 {
 	return &slots_list;
 }
@@ -645,9 +705,8 @@ int sysfs_enclosure_attached_to_cntrl(const char *path)
 	struct enclosure_device *device;
 
 	list_for_each(&enclo_list, device) {
-		if ((device->sysfs_path != NULL) &&
-		    (strncmp(device->sysfs_path, path, strlen(path)) == 0))
-			return 1;
+			if (strncmp(device->sysfs_path, path, strnlen(path, PATH_MAX)) == 0)
+				return 1;
 	}
 	return 0;
 }
