@@ -1,19 +1,20 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009-2019 Intel Corporation.
+ * Copyright (C) 2022-2023 Intel Corporation.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms and conditions of the GNU General Public License,
- * version 2, as published by the Free Software Foundation.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -26,6 +27,7 @@
 #include <limits.h>
 #include <regex.h>
 #include <stdarg.h>
+#include <stdbool.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <string.h>
@@ -34,6 +36,7 @@
 #include <sys/types.h>
 #include <time.h>
 #include <unistd.h>
+#include <assert.h>
 
 #if _HAVE_DMALLOC_H
 #include <dmalloc.h>
@@ -92,34 +95,27 @@ int get_bool(const char *path, int defval, const char *name)
 	return defval;
 }
 
-/*
- * Function returns 64-bit unsigned integer value read from a text file. See
- * utils.h for details.
- */
 uint64_t get_uint64(const char *path, uint64_t defval, const char *name)
 {
 	char *p = get_text(path, name);
-	uint64_t retval = defval;
 
-	if (p) {
-		if (sscanf(p, "%" SCNx64, &defval) == 1)
-			retval = defval;
-		free(p);
-	}
-	return retval;
+	if (!p)
+		return defval;
+
+	str_toul(&defval, p, NULL, 16);
+	free(p);
+	return defval;
 }
 
-/*
- * Function returns integer value read from a text file.
- * See utils.h for details.
- */
 int get_int(const char *path, int defval, const char *name)
 {
 	char *p = get_text(path, name);
-	if (p) {
-		defval = atoi(p);
-		free(p);
-	}
+
+	if (!p)
+		return defval;
+
+	str_toi(&defval, p, NULL, 10);
+	free(p);
 	return defval;
 }
 
@@ -181,11 +177,13 @@ ssize_t buf_write(const char *path, const char *buf)
 
 	if (path == NULL)
 		__set_errno_and_return(EINVAL);
-	if ((buf == NULL) || (strlen(buf) == 0))
+
+	if (buf == NULL || strnlen(buf, WRITE_BUFFER_SIZE) == 0)
 		__set_errno_and_return(ENODATA);
+
 	fd = open(path, O_WRONLY);
 	if (fd >= 0) {
-		size = write(fd, buf, strlen(buf));
+		size = write(fd, buf, strnlen(buf, WRITE_BUFFER_SIZE));
 		close(fd);
 	}
 	return size;
@@ -235,8 +233,8 @@ void get_id(const char *path, struct device_id *did)
 			t = strchr(p, ':');
 			if (t) {
 				*(t++) = '\0';
-				did->major = atoi(p);
-				did->minor = atoi(t);
+				str_toi(&did->major, p, NULL, 10);
+				str_toi(&did->minor, t, NULL, 10);
 			}
 			free(p);
 		}
@@ -359,6 +357,71 @@ char *str_dup(const char *src)
 	return ret;
 }
 
+static int _str_to_num(const char *strptr, char **endptr, int base, unsigned long *n, int is_signed)
+{
+	char *c;
+	int sign_occured = 0;
+
+	assert(n);
+	if (!strptr)
+		return 1;
+	errno = 0;
+	if (is_signed)
+		*n = (unsigned long)strtol(strptr, &c, base);
+	else {
+		while ((*strptr == '-' || *strptr == '+' || isspace(*strptr)) &&
+				*strptr != '\0' && !sign_occured) {
+			if (*strptr == '-' || *strptr == '+')
+				sign_occured = 1;
+			strptr++;
+		}
+		*n = strtoul(strptr, &c, base);
+	}
+	if (errno != 0 || strptr == c)
+		return 1;
+	if (endptr)
+		*endptr = c;
+	return 0;
+}
+
+static int _str_to_num_signed(const char *strptr, char **endptr, int base, unsigned long *n)
+{
+	return _str_to_num(strptr, endptr, base, n, 1);
+}
+
+static int _str_to_num_unsigned(const char *strptr, char **endptr, int base, unsigned long *n)
+{
+	return _str_to_num(strptr, endptr, base, n, 0);
+}
+
+static inline int value_in_range_signed(long value, long min, long max)
+{
+	return (value >= min && value <= max);
+}
+
+static inline int value_in_range_unsigned(unsigned long value, unsigned long min, unsigned long max)
+{
+	return (value >= min && value <= max);
+}
+
+#define _DECL_STR_TO_NUM(name, sign, type, min, max) \
+int name(sign type * dest, const char *strptr, char **endptr, int base) \
+{ \
+	unsigned long n; \
+	if ((!dest && !endptr) || _str_to_num_##sign(strptr, endptr, base, &n)) \
+		return 1; \
+	if (!value_in_range_##sign(n, min, max)) \
+		return 1; \
+	if (dest)\
+		*dest = (sign type)n; \
+	return 0; \
+}
+
+_DECL_STR_TO_NUM(str_tol, signed, long, LONG_MIN, LONG_MAX)
+_DECL_STR_TO_NUM(str_toul, unsigned, long, 0, ULONG_MAX)
+_DECL_STR_TO_NUM(str_toi, signed, int, INT_MIN, INT_MAX)
+_DECL_STR_TO_NUM(str_toui, unsigned, int, 0, UINT_MAX)
+
 char *get_path_hostN(const char *path)
 {
 	char *c = NULL, *s = NULL, *p = str_dup(path);
@@ -428,6 +491,7 @@ int match_string(const char *string, const char *pattern)
 	}
 
 	status = regexec(&regex, string, 0, NULL, 0);
+	regfree(&regex);
 	if (status != 0)
 		return 0;
 
@@ -443,7 +507,7 @@ int get_log_fd(void)
 
 void print_opt(const char *long_opt, const char *short_opt, const char *desc)
 {
-	printf("%-20s%-10s%s\n", long_opt, short_opt, desc);
+	printf("%-70s%-40s%s\n", long_opt, short_opt, desc);
 }
 
 /**
@@ -519,6 +583,13 @@ struct option longopt_all[] = {
 	[OPT_LIST_CTRL]    = {"list-controllers", no_argument, NULL, 'L'},
 	[OPT_LISTED_ONLY]  = {"listed-only", no_argument, NULL, 'x'},
 	[OPT_FOREGROUND]   = {"foreground", no_argument, NULL, '\0'},
+	[OPT_LIST_SLOTS]   = {"list-slots", no_argument, NULL, 'P'},
+	[OPT_GET_SLOT]     = {"get-slot", no_argument, NULL, 'G'},
+	[OPT_SET_SLOT]     = {"set-slot", no_argument, NULL, 'S'},
+	[OPT_CNTRL_TYPE]   = {"controller-type", required_argument, NULL, 'c'},
+	[OPT_DEVICE]       = {"device", required_argument, NULL, 'd'},
+	[OPT_SLOT]         = {"slot", required_argument, NULL, 'p'},
+	[OPT_STATE]        = {"state", required_argument, NULL, 's'},
 	[OPT_NULL_ELEMENT] = {NULL, no_argument, NULL, '\0'}
 };
 
@@ -622,9 +693,6 @@ status_t set_verbose_level(int log_level)
 
 const char *ibpi2str(enum ibpi_pattern ibpi)
 {
-#ifdef _TEST_CONFIG
-	return NULL;
-#else
 	static char buf[20];
 	const char *ret;
 
@@ -639,5 +707,70 @@ const char *ibpi2str(enum ibpi_pattern ibpi)
 	}
 
 	return ret;
-#endif
 }
+
+static const struct ibpi2value *get_ibpi2value(const unsigned int val,
+					       const struct ibpi2value *ibpi2val_arr,
+					       const int ibpi2value_arr_cnt,
+					       bool (*compar)(const unsigned int val,
+							      const struct ibpi2value *ibpi2val))
+{
+	int cnt = 0;
+	const struct ibpi2value *tmp = NULL;
+
+	do {
+		/* Prevent out of bound results */
+		assert(cnt < ibpi2value_arr_cnt);
+
+		tmp = ibpi2val_arr + cnt;
+		if (compar(val, tmp))
+			return tmp;
+		cnt++;
+	} while (tmp->ibpi != IBPI_PATTERN_UNKNOWN);
+
+	return tmp;
+}
+
+static bool compar_bits(const unsigned int val, const struct ibpi2value *ibpi2val)
+{
+	if (ibpi2val->value & val)
+		return true;
+	return false;
+}
+
+#define IBPI2VALUE_GET_FN(_name)							\
+const struct ibpi2value *get_by_##_name(const enum ibpi_pattern ibpi,			\
+					const struct ibpi2value *ibpi2val_arr,		\
+					int ibpi2value_arr_cnt)				\
+{											\
+	return get_ibpi2value(ibpi, ibpi2val_arr, ibpi2value_arr_cnt, compar_##_name);	\
+}
+
+#define COMPARE(_name)									\
+static bool compar_##_name(const unsigned int val, const struct ibpi2value *ibpi2val)	\
+{											\
+	if (ibpi2val->_name == val)							\
+		return true;								\
+	return false;									\
+}											\
+IBPI2VALUE_GET_FN(_name)								\
+
+IBPI2VALUE_GET_FN(bits);
+COMPARE(ibpi);
+COMPARE(value);
+/**
+ * @brief Get string from map.
+ *
+ * @param[in] 	scode		status code.
+ * @param[in] 	map		map.
+ * @return exit status if defined, NULL otherwise
+ */
+char *str_map(int scode, struct map *map)
+{
+	while(map->name) {
+		if (scode == map->value)
+			return map->name;
+		map++;
+	}
+	return NULL;
+}
\ No newline at end of file
