@@ -1,19 +1,20 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009-2019 Intel Corporation.
+ * Copyright (C) 2022-2023 Intel Corporation.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms and conditions of the GNU General Public License,
- * version 2, as published by the Free Software Foundation.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -29,6 +30,20 @@
 #include "syslog.h"
 #include "ibpi.h"
 
+struct map {
+	char *name;
+	int value;
+};
+
+/**
+ * @brief Get string from map.
+ *
+ * @param[in] 	scode		status code.
+ * @param[in] 	map		map.
+ * @return exit status if defined, NULL otherwise
+ */
+char *str_map(int scode, struct map *map);
+
 /**
  * Value is intentionally unused.
  * Macro to prevent compiler from raising unused result warning.
@@ -46,6 +61,11 @@
  */
 #define WRITE_BUFFER_SIZE   1024
 
+/**
+ * @brief Macro for compile time array size constants
+ */
+#define ARRAY_SIZE(x) (sizeof(x)/sizeof(x[0]))
+
 /**
  * This structure describes a device identifier. It consists of major and minor
  * attributes of device.
@@ -59,6 +79,14 @@ struct log_level_info {
 	int priority;
 };
 
+/**
+ * @brief The struct is used for mapping IPBI pattern to the controller internal value.
+ */
+struct ibpi2value {
+	unsigned int ibpi;
+	unsigned int value;
+};
+
 /**
  */
 #define PREFIX_DEBUG          "  DEBUG: "
@@ -307,6 +335,71 @@ char *str_cpy(char *dest, const char *src, size_t size);
  */
 char *str_dup(const char *src);
 
+/**
+ * @brief Converts string to long integer.
+ *
+ * This function works similar to strtol.
+ * Writes to destination only if successfully read the number.
+ * Destination can be NULL if endptr is set.
+ *
+ * @param[in]      dest            Pointer to destination.
+ * @param[in]      strptr          Pointer to source string.
+ * @param[in]      endptr          Pointer to the next character in string after parsed value.
+ * @param[in]      base            Base of the numerical value in string.
+ *
+ * @return 0 if operation was successful, otherwise 1.
+ */
+int str_tol(signed long *dest, const char *strptr, char **endptr, int base);
+
+/**
+ * @brief Converts string to unsigned long integer.
+ *
+ * This function works similar to strtoul. Skips + and - characters.
+ * Writes to destination only if successfully read the number.
+ * Destination can be NULL if endptr is set.
+ *
+ * @param[in]      dest            Pointer to destination.
+ * @param[in]      strptr          Pointer to source string.
+ * @param[in]      endptr          Pointer to the next character in string after parsed value.
+ * @param[in]      base            Base of the numerical value in string.
+ *
+ * @return 0 if operation was successful, otherwise 1.
+ */
+int str_toul(unsigned long *dest, const char *strptr, char **endptr, int base);
+
+/**
+ * @brief Converts string to integer.
+ *
+ * This function works similar to strtol, but for int.
+ * Writes to destination only if successfully read the number.
+ * Destination can be NULL if endptr is set.
+ *
+ * @param[in]      dest            Pointer to destination.
+ * @param[in]      strptr          Pointer to source string.
+ * @param[in]      endptr          Pointer to the next character in string after parsed value.
+ * @param[in]      base            Base of the numerical value in string.
+ *
+ * @return 0 if operation was successful, otherwise 1.
+ */
+int str_toi(signed int *dest, const char *strptr, char **endptr, int base);
+
+/**
+ * @brief Converts string to unsigned integer.
+ *
+ * This function works similar to strtoul, but for unsigned int.
+ * Skips + and - characters.
+ * Writes to destination only if successfully read the number.
+ * Destination can be NULL if endptr is set.
+ *
+ * @param[in]      dest            Pointer to destination.
+ * @param[in]      strptr          Pointer to source string.
+ * @param[in]      endptr          Pointer to the next character in string after parsed value.
+ * @param[in]      base            Base of the numerical value in string.
+ *
+ * @return 0 if operation was successful, otherwise 1.
+ */
+int str_toui(unsigned int *dest, const char *strptr, char **endptr, int base);
+
 /**
  */
 char *truncate_path_component_rev(const char *path, int index);
@@ -353,6 +446,13 @@ enum opt {
 	OPT_LIST_CTRL,
 	OPT_LISTED_ONLY,
 	OPT_FOREGROUND,
+	OPT_LIST_SLOTS,
+	OPT_GET_SLOT,
+	OPT_SET_SLOT,
+	OPT_CNTRL_TYPE,
+	OPT_DEVICE,
+	OPT_SLOT,
+	OPT_STATE,
 	OPT_NULL_ELEMENT
 };
 
@@ -364,4 +464,43 @@ status_t set_verbose_level(int log_level);
 
 const char *ibpi2str(enum ibpi_pattern ibpi);
 
+/**
+ * @brief Returns ibpi2value entry if IBPI matches
+ *
+ * @param[in]       ibpi                  IBPI pattern.
+ * @param[in]       ibpi2val_arr          IBPI pattern to value array.
+ * @param[in]       ibpi2value_arr_cnt    Array entries count.
+ *
+ * @return Corresponding ibpi2value entry last or entry with IBPI_PATTERN_UNKNOWN
+ */
+const struct ibpi2value *get_by_ibpi(const enum ibpi_pattern ibpi,
+				     const struct ibpi2value *ibpi2val_arr,
+				     int ibpi2value_arr_cnt);
+
+/**
+ * @brief Returns ibpi2value entry if value matches
+ *
+ * @param[in]       value                 value to compare.
+ * @param[in]       ibpi2val_arr          IBPI pattern to value array.
+ * @param[in]       ibpi2value_arr_cnt    Array entries count.
+ *
+ * @return Corresponding ibpi2value entry last or entry with IBPI_PATTERN_UNKNOWN
+ */
+const struct ibpi2value *get_by_value(const unsigned int value,
+				      const struct ibpi2value *ibpi2val_arr,
+				      int ibpi2value_arr_cnt);
+
+/**
+ * @brief Returns ibpi2value entry if any bit matches
+ *
+ * @param[in]       value                 value to compare.
+ * @param[in]       ibpi2val_arr          IBPI pattern to value array.
+ * @param[in]       ibpi2value_arr_cnt    Array entries count.
+ *
+ * @return Corresponding ibpi2value entry last or entry with IBPI_PATTERN_UNKNOWN
+ */
+const struct ibpi2value *get_by_bits(const unsigned int value,
+				     const struct ibpi2value *ibpi2val_arr,
+				     int ibpi2value_arr_cnt);
+
 #endif				/* _UTILS_H_INCLUDED_ */
